﻿using System.Xml;
using System.Xml.Linq;
using System.Collections.ObjectModel;
using Model.Stub;
using Microsoft.Maui.Storage;

namespace Model.Serialization;

public class LinqXmlSerialization : IDataManager
{
    private static string XMLPATH = Path.Combine(FileSystem.Current.AppDataDirectory, "Data");

    private static string XMLFILEPLAYLISTS = Path.Combine(XMLPATH, "playlists.xml");

    private static string XMLFILECUSTOMS = Path.Combine(XMLPATH, "customs.xml");


    private StubInfoTitle stubInfoTitle = new();

    public StubInfoTitle StubInfoTitle
    {
        get => stubInfoTitle;
    }

    private List<Artist> artists;

    public IEnumerable<Artist> Artists
    {
        get
        {
            return new List<Artist>(artists);
        }
    }

    private ObservableCollection<Album> albums;

    public ObservableCollection<Album> Albums
    {
        get
        {
            return new ObservableCollection<Album>(albums);
        }
    }

    private ObservableCollection<Playlist> playlists;

    public ObservableCollection<Playlist> Playlists
    {
        get
        {
            return new ObservableCollection<Playlist>(playlists);
        }
    }

    private ObservableCollection<InfoTitle> infoTitles;

    public ObservableCollection<InfoTitle> InfoTitles
    {
        get
        {
            return new ObservableCollection<InfoTitle>(infoTitles);
        }
    }

    private ObservableCollection<CustomTitle> customTitles;

    public ObservableCollection<CustomTitle> CustomTitles
    {
        get
        {
            return new ObservableCollection<CustomTitle>(customTitles);
        }
    }

    public LinqXmlSerialization()
    {
        playlists = new ObservableCollection<Playlist>();
        artists = new List<Artist>();
        infoTitles = new ObservableCollection<InfoTitle>();
        albums = new ObservableCollection<Album>();
        customTitles = new ObservableCollection<CustomTitle>();
        if (!Directory.Exists(XMLPATH))
        {
            Directory.CreateDirectory(XMLPATH);
        }
        Directory.SetCurrentDirectory(XMLPATH);
        LoadSerialization();
    }

    public void AddAlbum(Album album)
    {
        albums.Add(album);
    }

    public void AddArtist(Artist artist)
    {
        artists.Add(artist);
    }

    public void AddCustomTitle(CustomTitle title)
    {
        customTitles.Add(title);
    }

    public void AddInfoTitle(InfoTitle title)
    {
        infoTitles.Add(title);
    }

    public void AddPlaylist(Playlist playlist)
    {
        playlists.Add(playlist);
    }

    public ObservableCollection<Album> GetAlbums()
    {
        return albums;
    }

    public List<Artist> GetArtists()
    {
        return artists;
    }

    public ObservableCollection<CustomTitle> GetCustomTitles()
    {
        return customTitles;
    }

    public ObservableCollection<InfoTitle> GetInfoTitles()
    {
        return infoTitles;
    }

    public ObservableCollection<Playlist> GetPlaylists()
    {
        return playlists;
    }

    public void RemoveAlbum(Album album)
    {
        albums.Remove(album);
    }

    public void RemoveArtist(Artist artist)
    {
        artists.Remove(artist);
    }

    public void RemoveCustomTitle(CustomTitle title)
    {
        customTitles.Remove(title);
    }

    public void RemoveInfoTitle(InfoTitle title)
    {
        infoTitles.Remove(title);
    }

    public void RemovePlaylist(Playlist playlist)
    {
        playlists.Remove(playlist);
    }

    public void LoadSerialization()
    {
        LoadArtists();
        LoadAlbums();
        LoadInfoTitles();
        LoadCustomTitles();
        LoadPlaylists();
    }

    public void SaveSerialization()
    {
        SaveArtists();
        SaveCustomTitles();
        SaveInfoTitles();
        SavePlaylists();
        SaveAlbums();
    }

    public void LoadPlaylists()
    {
        if (!File.Exists(XMLFILEPLAYLISTS))
        {
            XDocument PlaylistFile = new XDocument();

            var playlist = playlists.Select(p => new XElement("Playlist",
                                                  new XAttribute("Name", p.Name),
                                                  new XElement("Description", p.Description),
                                                  new XElement("ImageURL", p.ImageURL),
                                                  new XElement("Titles", p.Titles.Any() ? p.Titles.Select(a => a.Name).Aggregate((playlistName, nextPlaylist) => playlistName + " " + nextPlaylist) : "")
                                                  ));

            PlaylistFile.Add(new XElement("Playlists", playlist));

            XmlWriterSettings settings = new XmlWriterSettings();
            settings.Indent = true;
            using (TextWriter tw = File.CreateText(XMLFILEPLAYLISTS))
            {
                using (XmlWriter writer = XmlWriter.Create(tw, settings))
                {
                    PlaylistFile.Save(writer);
                }
            }
        }
        XDocument PlaylistsFichier = XDocument.Load(XMLFILEPLAYLISTS);

        var playlists2 = PlaylistsFichier.Descendants("Playlist")
                                    .Select(eltPlaylist => new Playlist(
                                        eltPlaylist.Attribute("Name")!.Value,
                                        eltPlaylist.Element("Description")!.Value,
                                        eltPlaylist.Element("ImageURL")!.Value
                                    )).ToList();
        playlists = new ObservableCollection<Playlist>(playlists2);

        foreach (Playlist playlist in playlists)
        {
            var allCustoms = PlaylistsFichier.Descendants("Playlist")
                                                 .Single(ct => ct.Attribute("Name")?.Value == playlist.Name)
                                                 .Element("Titles")!.Value;

            if (allCustoms == null)
            {
                continue;
            }

            char[] separator = new char[] { ' ' };
            string[] customArray = allCustoms.Split(separator, StringSplitOptions.RemoveEmptyEntries);
            List<string> customsList = customArray.ToList();

            foreach(var custom in customsList)
            {
                CustomTitle customTitle = GetCustomTitleByPath(custom);

                if (customTitle == null)
                {
                    continue;
                }

                playlist.AddTitle(customTitle);
            }
        }
    }

    public void SavePlaylists()
    {
        XDocument PlaylistsFichier = new XDocument();

        var playlist = playlists.Select(playlist => new XElement("Playlist",
                                                    new XAttribute("Name", playlist.Name),
                                                    new XElement("Description", playlist.Description),
                                                    new XElement("ImageURL", playlist.ImageURL),
                                                    new XElement("Titles", playlist.Titles.Any() ? playlist.Titles.Select(p => p.Path).Aggregate((playlistName, nextPlaylist) => playlistName + " " + nextPlaylist) : "")
                                                    ));
        PlaylistsFichier.Add(new XElement("Playlists", playlist));

        XmlWriterSettings settings = new XmlWriterSettings();
        settings.Indent = true;

        using(TextWriter tw = File.CreateText(XMLFILEPLAYLISTS))
        {
            using(XmlWriter writer  = XmlWriter.Create(tw, settings))
            {
                PlaylistsFichier.Save(writer);
            }
        }
    }
        
    public void LoadArtists()
    {
        artists = StubInfoTitle.StubAlbum.StubArtist.GetArtists();
    }

    public void SaveArtists()
    {
        // Don't do anything because it's static data
    }

    public void LoadCustomTitles()
    {
        if (!File.Exists(XMLFILECUSTOMS))
        {

            XDocument CustomsFile2 = new XDocument();

            var custom = customTitles.Select(c => new XElement("CustomTitle",
                                                  new XAttribute("Name", c.Name),
                                                  new XElement("ImageURL", c.ImageURL),
                                                  new XElement("Information", c.Information),
                                                  new XElement("Path",c.Path)
                                                  ));

            CustomsFile2.Add(new XElement("Customs", custom));

            XmlWriterSettings settings = new XmlWriterSettings();
            settings.Indent = true;
            using (TextWriter tw = File.CreateText(XMLFILECUSTOMS))
            {
                using (XmlWriter writer = XmlWriter.Create(tw, settings))
                
                    CustomsFile2.Save(writer);
            }
        }
        XDocument CustomsFile = XDocument.Load(XMLFILECUSTOMS);

        var customTitles2 = CustomsFile.Descendants("CustomTitle")
                                    .Select(eltPlaylist => new CustomTitle(
                                        eltPlaylist.Attribute("Name")!.Value,
                                        eltPlaylist.Element("ImageURL")!.Value,
                                        eltPlaylist.Element("Information")!.Value,
                                        eltPlaylist.Element("Path")!.Value
                                    )).ToList();
        customTitles = new ObservableCollection<CustomTitle>(customTitles2);
    }

    public void SaveCustomTitles()
    {
        XDocument CustomsFile = new XDocument();

        var customs = customTitles.Select(custom => new XElement("CustomTitle",
                                                    new XAttribute("Name", custom.Name),
                                                    new XElement("ImageURL", custom.ImageURL),
                                                    new XElement("Information", custom.Information),
                                                    new XElement("Path", custom.Path)
                                                    ));

        CustomsFile.Add(new XElement("Customs", customs));

        XmlWriterSettings settings = new XmlWriterSettings();
        settings.Indent = true;

        using (TextWriter tw = File.CreateText(XMLFILECUSTOMS))
        {
            using (XmlWriter writer = XmlWriter.Create(tw, settings))
            {
                CustomsFile.Save(writer);
            }
        }
    }

    public void LoadAlbums()
    {
        albums = StubInfoTitle.StubAlbum.GetAlbums();
    }

    public void SaveAlbums()
    {
        // Don't do anything because it's static data
    }

    public void LoadInfoTitles()
    {
        infoTitles = StubInfoTitle.GetInfoTitles();
        foreach (var infoTitle in InfoTitles)
        {
            GetAlbumById(infoTitle.AlbumID)?.AddTitle(infoTitle); 
        }
    }

    public void SaveInfoTitles()
    {
        // Don't do anything because it's static data
    }

    public static Genre GetGenreByName(string genre)
    {
        if (genre == "HIP_HOP") return Genre.HIP_HOP;
        if (genre == "POP") return Genre.POP;
        if (genre == "ROCK") return Genre.ROCK;
        if (genre == "ELECTRO") return Genre.ELECTRO;
        if (genre == "CLASSIQUE") return Genre.CLASSIQUE;
        if (genre == "JAZZ") return Genre.JAZZ;
        if (genre == "VARIETE_FRANCAISE") return Genre.VARIETE_FRANCAISE;
        if (genre == "VARIETE_INTERNATIONALE") return Genre.VARIETE_INTERNATIONALE;
        if (genre == "REGGAE") return Genre.REGGAE;
        if (genre == "RAP") return Genre.RAP;
        if (genre == "RNB") return Genre.RNB;
        if (genre == "DISCO") return Genre.DISCO;
        if (genre == "BLUES") return Genre.BLUES;
        if (genre == "COUNTRY") return Genre.COUNTRY;
        if (genre == "FUNK") return Genre.FUNK;
        if (genre == "GOSPEL") return Genre.GOSPEL;
        if (genre == "METAL") return Genre.METAL;
        else return Genre.K_POP;
    }

    public InfoTitle GetInfoTitleByName(string name)
    {
        foreach(InfoTitle it in infoTitles)
        {
            if (it.Name == name)
            {
                return it;
            }
        }
        return null;
    }

    public Artist GetArtistByName(string name)
    {
        foreach(Artist a in artists)
        {
            if (a.Name == name)
            {
                return a;
            }
        }
        return null;
    }

    public Album GetAlbumByName(string name)
    {
        foreach(Album a in albums)
        {
            if (a.Name == name)
            {
                return a;
            }
        }
        return null;
    }

    public Album GetAlbumById(long id)
    {
        foreach(Album a in albums)
        {
            if (a.ID == id) return a;
        }
        return null;
    }

    public CustomTitle GetCustomTitleByPath(string custom)
    {
        foreach(CustomTitle customTitle in customTitles)
        {
            if(customTitle.Path == custom)
            {
                return customTitle;
            }
        }
        return null;
    }

    public void AddAlbums(List<Album> albumsList)
    {
        foreach(Album a in albumsList)
        {
            albums.Add(a);
        }
    }

    public void AddArtists(List<Artist> artistsList)
    {
        foreach (Artist a in artistsList)
        {
            artists.Add(a);
        }
    }

    public void AddPlaylists(List<Playlist> playlistsList)
    {
        foreach (Playlist p in playlistsList)
        {
            playlists.Add(p);
        }
    }

    public void AddCustomTitles(List<CustomTitle> customTitlesList)
    {
        foreach (CustomTitle ct in customTitlesList)
        {
            customTitles.Add(ct);
        }
    }

    public void AddInfoTitles(List<InfoTitle> infoTitlesList)
    {
        foreach (InfoTitle it in infoTitlesList)
        {
            infoTitles.Add(it);
        }
    }

    public Playlist GetPlaylistByName(string name)
    {
        foreach(Playlist p in playlists)
        {
            if (p.Name == name)
            {
                return p;
            }
        }
        return null;
    }

    public void UpdateCustomTitle(CustomTitle title, string name, string url, string info, string path)
    {
        title.Name = name;
        title.ImageURL = url;
        title.Information = info;
        title.Path = path;
    }

    public void UpdateCustomTitleByPath(string path, string name, string newUrl, string info, string newPath)
    {
        CustomTitle title = GetCustomTitleByPath(path);
        if (title != null)
        {
            title.Name = name;
            title.ImageURL = newUrl;
            title.Information = info;
            title.Path = newPath;
        }
    }

    public void UpdateInfoTitle(InfoTitle title, string name, string url, string info, Artist artist, string description, Genre genre)
    {
        title.Name = name;
        title.ImageURL = url;
        title.Information = info;
        title.Description = description;
        title.Genre = genre;
    }

    public void UpdateInfoTitleByName(string name, string newUrl, string info, Artist artist, string description, Genre genre)
    {
        InfoTitle title = GetInfoTitleByName(name);
        if (title != null)
        {
            title.Name = name;
            title.ImageURL = newUrl;
            title.Information = info;
            title.Description = description;
            title.Genre = genre;
        }
    }

    public void UpdateAlbum(Album album, string name, string url, Artist artist, string description, string info)
    {
        album.Name = name;
        album.ImageURL = url;
        album.Artist = artist;
        album.Description = description;
        album.Information = info;
    }

    public void UpdateAlbumByName(string name, string newUrl, Artist artist, string description, string info)
    {
        Album album = GetAlbumByName(name);
        if (album != null)
        {
            album.Name = name;
            album.ImageURL = newUrl;
            album.Artist = artist;
            album.Description = description;
            album.Information= info;
        }
    }

    public void UpdateAlbumByArtistName(Album album, string name, string url, string artist, string description, string info)
    {
        album.Name = name;
        album.ImageURL = url;
        Artist artist2 = GetArtistByName(artist);
        if (artist2 != null)
        {
            album.Artist = artist2;
        }
        album.Description = description;
        album.Information = info;
    }

    public void UpdateAlbumByNameByArtistName(string name, string newUrl, string artist, string description, string info)
    {
        Album album = GetAlbumByName(name);
        if (album != null)
        {
            album.Name = name;
            album.ImageURL = newUrl;
            Artist artist2 = GetArtistByName(artist);
            if (artist2 != null)
            {
                album.Artist = artist2;
            }
            album.Description = description;
            album.Information = info;
        }
    }

    public void UpdatePlaylist(Playlist playlist, string name, string description, string url)
    {
        playlist.Name = name;
        playlist.Description = description;
        playlist.ImageURL = url;
    }

    public void UpdatePlaylistByName(string name, string description, string newUrl)
    {
        Playlist playlist = GetPlaylistByName(name);
        if (playlist != null)
        {
            playlist.Name = name;
            playlist.Description = description;
            playlist.ImageURL = newUrl;
        }
    }

    public void UpdateArtist(Artist artist, string name)
    {
        artist.Name = name;
    }

    public void UpdateArtistByName(string name, string newName)
    {
        Artist artist = GetArtistByName(newName);
        if (artist != null)
        {
            artist.Name = newName;
        }
    }

    public void RemoveAlbums(List<Album> albumsList)
    {
        foreach (Album album in albumsList)
        {
            albums.Remove(album);
        }
    }

    public void RemoveArtists(List<Artist> artistsList)
    {
        foreach(Artist artist in artistsList)
        {
            artists.Remove(artist);
        }
    }

    public void RemovePlaylists(List<Playlist> playlistsList)
    {
        foreach (Playlist playlist in playlistsList)
        {
            playlists.Remove(playlist);
        }
    }

    public void RemoveCustomTitles(List<CustomTitle> customTitlesList)
    {
        foreach(CustomTitle customTitle in customTitlesList)
        {
            customTitles.Remove(customTitle);
        }
    }

    public void RemoveInfoTitles(List<InfoTitle> infoTitlesList)
    {
        foreach (InfoTitle infoTitle in infoTitlesList)
        {
            infoTitles.Remove(infoTitle);
        }
    }

    public bool ExistsPlaylist(Playlist playlist)
    {
        foreach(Playlist p in playlists)
        {
            if (p ==  playlist)
            {
                return true;
            }
        }
        return false;
    }

    public bool ExistsPlaylistByName(string name)
    {
        foreach(Playlist p in playlists)
        {
            if (p.Name == name)
            {
                return true;
            }
        }
        return false;
    }

    public bool ExistsAlbum(Album album)
    {
        foreach(Album a in albums)
        {
            if(a == album)
            {
                return true;
            }
        }
        return false;
    }

    public bool ExistsAlbumByName(string name)
    {
        foreach(Album a in albums)
        {
            if (a.Name == name)
            {
                return true;
            }
        }
        return false;
    }

    public bool ExistsArtist(Artist artist)
    {
        foreach (Artist a in artists)
        {
            if (a == artist)
            {
                return true;
            }
        }
        return false;
    }

    public bool ExistsArtistByName(string name)
    {
        foreach(Artist a in artists)
        {
            if(a.Name == name)
            {
                return true;
            }
        }
        return false;
    }

    public bool ExistsCustomTitle(CustomTitle title)
    {
        foreach(CustomTitle ct in customTitles)
        {
            if(title == ct)
            {
                return true;
            }
        }
        return false;
    }

    public bool ExistsCustomTitleByName(string name)
    {
        foreach(CustomTitle ct in customTitles)
        {
            if(ct.Name == name)
            {
                return true;
            }
        }
        return false;
    }

    public bool ExistsInfoTitle(InfoTitle title)
    {
        foreach(InfoTitle it in infoTitles)
        {
            if(it == title)
            {
                return true;
            }
        }
        return false;
    }

    public bool ExistsInfoTitleByName(string name)
    {
        foreach(InfoTitle it in infoTitles)
        {
            if(it.Name == name)
            {
                return true;
            }
        }
        return false;
    }

}
